// Copyright 2006 all rights reserved; see LICENSE file for BSD-style license

package edu.berkeley.sbp;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.Sequence.Position;
import java.io.*;
import java.util.*;

/** if ambiguity checking is enabled, this exception is thrown to signal that the parse was ambiguous */
public class Ambiguous extends Exception {

    final Forest<?> ambiguity;
    private final HashSet<Tree<?>> ht;

    /**
     *  @param ht a specially-constructed set of trees with shared nodes replaced by '*'
     */
    Ambiguous(Forest<?> ambiguity, HashSet<Tree<?>> ht) {
        this.ambiguity = ambiguity;
        this.ht = ht;
    }

    public Forest<?> getAmbiguity() { return ambiguity; }

    /** WARNING: this method is not considered part of the "stable API"; it may be removed in the future */
    public Input.Region getRegion() { return ambiguity.getRegion(); }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("unresolved ambiguity at "+ambiguity.getRegion()+"; shared subtrees are shown as \"*\" ");
        for(Tree<?> result : ht) {
            sb.append("\n    possibility: ");
            StringBuffer sb2 = new StringBuffer();
            result.toPrettyString(sb2);
            sb.append(StringUtil.indent(sb2.toString(), 15));
        }
        return sb.toString();
    }
}
